"use strict";Object.defineProperty(exports, "__esModule", { value: true });exports.TableFilterQueryWithAggregate = exports.TableFilterQuery = undefined;var _extends = Object.assign || function (target) {for (var i = 1; i < arguments.length; i++) {var source = arguments[i];for (var key in source) {if (Object.prototype.hasOwnProperty.call(source, key)) {target[key] = source[key];}}}return target;};var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {return typeof obj;} : function (obj) {return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;};var _lodash = require("lodash");var _lodash2 = _interopRequireDefault(_lodash);
var _async = require("async");var _async2 = _interopRequireDefault(_async);
var _moment = require("moment");var _moment2 = _interopRequireDefault(_moment);
var _settings = require("./settings");function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { default: obj };}function _defineProperty(obj, key, value) {if (key in obj) {Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true });} else {obj[key] = value;}return obj;}

var utils = {
    removeExtraTableParams: function removeExtraTableParams(obj) {
        var x = Object.assign({}, obj);
        delete x.results;
        delete x.page;
        delete x.count;
        delete x.sortField;
        delete x.sortOrder;
        delete x.selectors;
        delete x.select;
        delete x.regExFilters;
        delete x.populateArr;
        delete x.project;
        delete x.dateFilter;

        return x;
    },
    runTableDataQuery: function runTableDataQuery(model, options) {
        return new Promise(function (resolve) {var

            results = options.results,page = options.page,sortField = options.sortField,sortOrder = options.sortOrder,selectors = options.selectors;
            var filters = utils.removeExtraTableParams(options);

            var query = model.find({});
            var countQuery = model.count();

            _lodash2.default.each(filters, function (val, key) {
                if (val.length && (typeof val === "undefined" ? "undefined" : _typeof(val)) === 'object') {
                    if (val.length === 1) {
                        _lodash2.default.each(val, function (item) {
                            query.where(_defineProperty({}, key, new RegExp(item, 'ig')));
                            countQuery.where(_defineProperty({}, key, new RegExp(item, 'ig')));
                        });
                    } else {
                        query.where(_defineProperty({}, key, { $in: val }));
                        countQuery.where(_defineProperty({}, key, { $in: val }));
                    }
                }
            });

            // selectors

            if (results) {
                query.limit(results);
                query.skip((page - 1) * results);
            }
            if (sortField) query.sort(_defineProperty({}, sortField, sortOrder === "ascend" ? 'asc' : 'desc'));

            query.exec(function (err, data) {
                countQuery.exec(function (er, count) {
                    resolve({ data: data, count: count });
                });
            });
        });
    } };


function isJson(str) {
    try {
        JSON.parse(str);
    } catch (e) {
        return false;
    }
    return true;
}

var TableFilterQuery = exports.TableFilterQuery = function TableFilterQuery(Model, Params) {
    return new Promise(function (resolve) {var _Params$results =

        Params.results,results = _Params$results === undefined ? 10 : _Params$results,_Params$page = Params.page,page = _Params$page === undefined ? 1 : _Params$page,sortField = Params.sortField,sortOrder = Params.sortOrder,_Params$regExFilters = Params.regExFilters,regExFilters = _Params$regExFilters === undefined ? [] : _Params$regExFilters,select = Params.select,populateArr = Params.populateArr;

        var populateArrFilters = [];


        var filter = utils.removeExtraTableParams(Params);var

        dateFilter = Params.dateFilter;var
        customQuery = filter.customQuery;

        results = parseInt(results);
        page = parseInt(page);

        var query = Model.find({}, {});
        var countQuery = Model.count({});


        if (populateArr) {
            _lodash2.default.each(populateArr, function (val, key) {
                /* console.log(val)
                                                                      let x = val.path
                                                                     if (filter && filter[val.path]) {
                                                                         val.match = {[val.select]: new RegExp(filter[val.path][0], 'ig')}
                                                                         delete filter[val.path]
                                                                     }
                                                                      console.log(val)*/


                query.populate(val);
            });
        }

        if (customQuery) {
            if (typeof customQuery === "string" && isJson(customQuery)) {
                customQuery = JSON.parse(customQuery);
            }
            query.where(_extends({}, customQuery));
            countQuery.where(_extends({}, customQuery));
            delete filter.customQuery;
        }

        if (filter) {
            _lodash2.default.each(filter, function (val, key) {

                if (isJson(val)) {
                    val = JSON.parse(val);
                    query.where(_defineProperty({}, key, val));
                    countQuery.where(_defineProperty({}, key, val));
                } else {
                    if (val !== undefined) {
                        var valueWord = val;
                        if (regExFilters.includes(key)) {
                            valueWord = new RegExp(val, 'ig');
                        }
                        query.where(_defineProperty({}, key, valueWord));
                        countQuery.where(_defineProperty({}, key, valueWord));
                    }
                }


            });
        }

        if (dateFilter) {
            dateFilter = JSON.parse(dateFilter);

            if (dateFilter.from && dateFilter.to) {
                query.where(_defineProperty({}, dateFilter.key, { $gte: (0, _moment2.default)(dateFilter.from).startOf('day') }));
                query.where(_defineProperty({}, dateFilter.key, { $lte: (0, _moment2.default)(dateFilter.to).endOf('day') }));


                countQuery.where(_defineProperty({}, dateFilter.key, { $gte: (0, _moment2.default)(dateFilter.from).startOf('day') }));
                countQuery.where(_defineProperty({}, dateFilter.key, { $lte: (0, _moment2.default)(dateFilter.to).endOf('day') }));
            }

        }


        if (select) {
            query.select(select);
        }
        if (sortField) {

            var order = sortOrder === 'ascend' ? 'asc' : 'desc';

            query.sort(_defineProperty({}, sortField, order));
        }

        query.skip((page - 1) * results).limit(results);
        query.lean().exec(function (err, data) {
            //console.log(err)
            if (err) {
                return resolve(_extends({}, _settings.errorObj));

            } else {
                countQuery.exec(function (err, result) {
                    // console.log(data[3])
                    resolve(_extends({ data: data, total: result }, _settings.successObj));
                });
            }
        });

    });
};
var TableFilterQueryWithAggregate = exports.TableFilterQueryWithAggregate = function TableFilterQueryWithAggregate(Model, fieldName, Params) {
    return new Promise(function (resolve) {var _Params$count =

        Params.count,count = _Params$count === undefined ? 10 : _Params$count,_Params$page2 = Params.page,page = _Params$page2 === undefined ? 1 : _Params$page2,sortField = Params.sortField,sortOrder = Params.sortOrder,_Params$regExFilters2 = Params.regExFilters,regExFilters = _Params$regExFilters2 === undefined ? [] : _Params$regExFilters2,project = Params.project,populateArr = Params.populateArr;

        var filter = utils.removeExtraTableParams(Params);


        count = parseInt(count);
        page = parseInt(page);


        var matchArr = [];


        var query = Model.aggregate([{ $unwind: "$" + ("" + fieldName) }, { $project: project }]);

        var countQuery = Model.aggregate([{ $unwind: "$" + ("" + fieldName) }]);


        if (filter) {
            _lodash2.default.each(filter, function (val, key) {
                if (val !== undefined) {
                    var valueWord = val;

                    if (regExFilters.includes(key)) {
                        valueWord = new RegExp(val, 'ig');
                    }

                    if (Params.project) {
                        var findPro = _lodash2.default.find(Params.project, function (item1, key1) {
                            return key1.indexOf('.' + key) >= 0;
                        });
                        if (findPro) {
                            var newKey = findPro[0] ? findPro[0] : fieldName;
                            key = newKey + "." + key;
                        }
                    } else {
                        key = fieldName + "." + key;
                    }

                    matchArr.push(_defineProperty({}, key, valueWord));

                }


            });
        }

        if (matchArr.length) {
            query.match({ $and: matchArr });

        }


        query.skip((page - 1) * count).limit(count);


        if (sortField) {
            var order = sortOrder === 'ascend' ? 'asc' : 'desc';
            query.sort(_defineProperty({}, sortField, order));
        }


        query.exec(function (err, data) {


            if (err) {
                return resolve(_extends({}, _settings.errorObj, { data: [], total: 0 }));
            } else {


                if (matchArr.length) {
                    countQuery.match({ $and: matchArr });

                }
                countQuery.group({
                    _id: null,
                    count: { $sum: 1 } });



                //resolve({data: data, total: data.length, ...successObj})

                countQuery.exec(function (couerr, coudoc) {
                    if (coudoc) {
                        coudoc = coudoc[0];
                    }
                    var dataArr = Object.assign([], data);
                    _async2.default.each(populateArr, function (item, cb) {
                        var ModalName = item.schemaName;
                        ModalName.populate(dataArr, {
                            path: item.path,
                            select: item.select },
                        function (err, result1) {
                            if (!err) {
                                dataArr = Object.assign([], result1);
                                cb();
                            } else {
                                cb();
                            }
                        });
                    }, function () {
                        resolve(_extends({ data: dataArr, total: coudoc ? coudoc.count : 0 }, _settings.successObj));
                    });


                });
            }
        });

    });
};exports.default =

utils;